<?php

namespace App\Console\Commands;

use App\Mail\AccountReceivableDeadlineReminderMail;
use App\Models\AccountReceivable;
use App\Models\Supplier;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Mail;
use App\Services\NotificationsService;

class AccountReceivableDeadlineReminderCash extends Command
{
    /** @var NotificationsService */
    private $notificationsService;

    /**
     * The name and signature of the console command.
     *
     * @var string
     */

    public function __construct(NotificationsService $notificationsService)
    {
        parent::__construct();
        $this->notificationsService = $notificationsService;
    }

    protected $signature = 'deadlineReminder:AccountReceivableCash';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Recordatorio de fecha de vencimiento de cuenta por pagar al contado';

    /**
     * Execute the console command.
     */
    public function handle()
    {
        $notifiedCustomers = [];

        $suppliers = Supplier::whereHas('tickets', function ($q) {
            $q->whereHas('wayToPay', fn($q) => $q->where('SII_fmapago', 1)) 
              ->whereHas('accountReceivable', function ($q) {
                  $q->where('paid', 0)
                    ->active()
                    ->where('deadline', '<', Carbon::now()->toDateString());
              });
        })
        ->where('email', '!=', null)
        ->active()
        ->get(); 

        foreach ($suppliers as $supplier) {
            $accounts = AccountReceivable::whereHas(
                'ticket',
                fn($q) => $q->where('idcustomer', $supplier->id)
                    ->active()
            )
                ->where('deadline', '<', Carbon::now()->toDateString())
                ->where('paid', 0)
                ->active()
                ->get()
                ->load('ticket');

            Mail::to($supplier->email)
                ->send(new AccountReceivableDeadlineReminderMail(
                    $supplier,
                    $accounts
                ));

            if ($accounts->isNotEmpty()) {
                $ids_receiver = User::whereHas(
                    'detailUser',
                    fn($q) => $q->where('iduser_type', 2)
                        ->where('idcompany', $accounts->first()->ticket->idcompany)
                )->active()->pluck('id')->toArray();

                # URL de redirección
                $data_json = (object) [
                    'url' => '/cartola'
                ];

                $messageBody = "De acuerdo a nuestros registros, el cliente " .
                    ($supplier->rut ?? '') . " " .
                    ($supplier->name_rz ?? '') .
                    " mantiene pendiente o se encuentran por vencer compromisos de pago 
                que deben ser regularizados.";


                $newNotification = [
                    'message_title' => 'Plazo de pago expirado',
                    'message_body'  => $messageBody,
                    'data_json'     => $data_json,
                    'idcompany'     => $accounts->first()->ticket->idcompany,
                    'ids_receiver'  => $ids_receiver,
                ];

                $this->notificationsService->create($newNotification);

                $notifiedCustomers[] = $supplier->id;
            }
        }
    }
}
