<?php

namespace App\Exports;

use Carbon\Carbon;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\WithStyles;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Events\AfterSheet;

class CartolaExport implements FromCollection, WithStyles, WithEvents, WithHeadings
{
    protected $data;
    protected $typesNames = [
        '1' => 'Factura emitida',
        '2' => 'Abono/Anticipo recibido',
        '3' => 'Pago de factura',
        '4' => 'Anulación de pago',
    ];

    public function __construct($data)
    {
        $this->data = $data;
    }
    /**
     * @return \Illuminate\Support\Collection
     */
    public function collection()
    {
        # Retornamos una colección vacía porque cargaremos manualmente los datos en AfterSheet
        return collect([]);
    }
    public function headings(): array
    {
        return [
            [
                '',
                'Desde',
                $this->data['start_date'],
                'Hasta',
                $this->data['end_date']

            ],
            [],
        ];
    }

    public function styles(Worksheet $sheet)
    {
        $sheet->getStyle('B1')->applyFromArray([
            'font' => ['bold' => true, 'color' => ['rgb' => 'FFFFFF']],
            'fill' => [
                'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                'color' => ['rgb' => '69ABC3']
            ],
            'alignment' => [
                'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER
            ]
        ]);

        $sheet->getStyle('D1')->applyFromArray([
            'font' => ['bold' => true, 'color' => ['rgb' => 'FFFFFF']],
            'fill' => [
                'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                'color' => ['rgb' => '69ABC3']
            ],
            'alignment' => [
                'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER
            ]
        ]);

        $sheet->getStyle('B1:E1')->applyFromArray([
            'borders' => [
                'allBorders' => [
                    'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                    'color' => ['argb' => '000000'],
                ],
            ],
        ]);

        # Ajuste de tamaño automático de columnas
        foreach (range('A', 'H') as $columnID) {
            $sheet->getColumnDimension($columnID)->setAutoSize(true);
        }
    }

    /**
     * Evento AfterSheet para cargar datos manualmente y ajustar la posición
     */
    public function registerEvents(): array
    {

        return [
            AfterSheet::class => function (AfterSheet $event) {
                $sheet = $event->sheet;

                # Insertar una fila al inicio para el título
                $sheet->insertNewRowBefore(1, 1);

                # Fusionar celdas para el título y centrarlo
                $sheet->setCellValue('A1', $this->data['name_rz'] . ' - ' . $this->data['rut']);
                $sheet->mergeCells('A1:H1');
                $sheet->getStyle('A1')->applyFromArray([
                    'font' => [
                        'bold' => true,
                        'size' => 14,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                ]);

                # Fila inicial para la tabla de detalles
                $startRow = 4;

                # Encabezado de tabla de detalles
                $sheet->setCellValue("A{$startRow}", 'Fecha');
                $sheet->setCellValue("B{$startRow}", 'Movimiento');
                $sheet->setCellValue("C{$startRow}", 'Folio');
                $sheet->setCellValue("D{$startRow}", 'Cargo');
                $sheet->setCellValue("E{$startRow}", 'Abono');
                $sheet->setCellValue("F{$startRow}", 'Saldo');
                $sheet->setCellValue("G{$startRow}", 'Método de pago');
                $sheet->setCellValue("H{$startRow}", 'Glosa');

                # Estilo para encabezado de tabla de detalles
                $sheet->getStyle("A{$startRow}:H{$startRow}")->applyFromArray([
                    'font' => ['bold' => true, 'color' => ['rgb' => 'FFFFFF']],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['rgb' => '69ABC3']
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER
                    ]
                ]);

                # Insertar datos de cuentas por cobrar
                $currentRow = $startRow + 1;
                foreach ($this->data['details'] as $detail) {
                    $sheet->setCellValue("A{$currentRow}", Carbon::parse($detail['ticket']['created_at'])->format('Y-m-d H:i') ?? '-');
                    $sheet->setCellValue("B{$currentRow}", $this->typesNames[$detail['type']] ?? '-');
                    $sheet->setCellValue("C{$currentRow}", $detail['ticket']['folio'] ?? '-');
                    $sheet->setCellValue("D{$currentRow}", $detail['charge'] ?? '0');
                    $sheet->setCellValue("E{$currentRow}", $detail['payment'] ?? '0');
                    $sheet->setCellValue("F{$currentRow}", $detail['balance'] ?? '0');
                    $sheet->setCellValue("G{$currentRow}", $detail['wayToPay']['name'] ?? '-');
                    $sheet->setCellValue("H{$currentRow}", $detail['gloss']);

                    $sheet->setCellValue("A{$currentRow}", \PhpOffice\PhpSpreadsheet\Shared\Date::stringToExcel(Carbon::parse($detail['ticket']['created_at'])->format('Y-m-d H:i')));
                    $sheet->getStyle("A{$currentRow}")
                        ->getNumberFormat()
                        ->setFormatCode('dd/mm/yyyy hh:mm');

                    $currentRow++;
                }

                $sheet->getStyle("D" . ($startRow + 1) . ":F" . ($currentRow - 1))
                    ->getNumberFormat()
                    ->setFormatCode('"$" #,##0');

                # Bordes para tabla de detalles
                $sheet->getStyle("A{$startRow}:H" . ($currentRow - 1))->applyFromArray([
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '000000'],
                        ],
                    ],
                ]);

                # Formato de fecha para el periodo
                $sheet->setCellValue("C2", \PhpOffice\PhpSpreadsheet\Shared\Date::stringToExcel(Carbon::parse($this->data['start_date'])->format('Y-m-d H:i')));
                $sheet->getStyle("C2")
                    ->getNumberFormat()
                    ->setFormatCode('dd/mm/yyyy');

                $sheet->setCellValue("E2", \PhpOffice\PhpSpreadsheet\Shared\Date::stringToExcel(Carbon::parse($this->data['start_date'])->format('Y-m-d H:i')));
                $sheet->getStyle("E2")
                    ->getNumberFormat()
                    ->setFormatCode('dd/mm/yyyy');
            }
        ];
    }
}
