<?php

namespace App\Http\Resources;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Http\Resources\Json\JsonResource;

class AccountReceivableResource extends JsonResource
{
    /**
     * Transform the resource into an array.
     *
     * @return array<string, mixed>
     */
    public function toArray(Request $request): array
    {
        $ticket = $this->relationLoaded('ticket') ? $this->whenLoaded('ticket') : null;
        $accountReceivableDets = $this->relationLoaded('accountReceivableDets') ? $this->whenLoaded('accountReceivableDets') : null;

        return [
            'id' => $this->id,
            'idticket' => $this->idticket,
            $this->mergeWhen($ticket, fn() => [
                'way_to_pay' => $ticket->wayToPay->name,
                'folio' => $ticket->folio,
                'customer' => $ticket->customer,
                'client_address' => $ticket->clientAddress,
                'ticket_date' => $ticket->created_at ? Carbon::parse($ticket->created_at)->format('Y-m-d H:i') : '',
                'type_document' => $ticket->typeDocument->name
            ]),
            'deadline' => $this->deadline,
            'amount_paid' => $this->amount_paid,
            'debt' => (int) $this->debt,
            'total' => (int) $this->total,
            'payment_date' => $this->payment_date,
            'paid' => (int) $this->paid,
            'status' => $this->status,
            'url_document' => $this->url_document,
            $this->mergeWhen($accountReceivableDets, fn() => [
                'accountReceivableDets' => $accountReceivableDets
            ]),
            $this->mergeWhen(request()->boolean('withRowColor'), function () {
                return [
                    'row_color' => $this->getRowColor($this->deadline)
                ];
            }),

        ];
    }

    private function getRowColor($deadline)
    {
        $now = Carbon::now()->toDateString();

        if ($now > $deadline) {
            return '#FFDAB9';
        } else {
            return 'inherit';
        }
    }
}
