<?php

namespace App\Http\Resources;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Http\Resources\Json\JsonResource;

class CartolaResource extends JsonResource
{
    /**
     * Transform the resource into an array.
     *
     * @return array<string, mixed>
     */
    public function toArray(Request $request): array
    {
        $supplier = $this->relationLoaded('supplier') ? $this->whenLoaded('supplier') : null;
        $ticket = $this->relationLoaded('ticket') ? $this->whenLoaded('ticket') : null;
        $wayToPay = $this->relationLoaded('wayToPay') ? $this->whenLoaded('wayToPay') : null;
        $paymentVoucher = $this->relationLoaded('paymentVoucher') ? $this->whenLoaded('paymentVoucher') : null;

        return [
            'id' => $this->id,
            'correlative' => $this->correlative,
            'idsupplier' => $this->idsupplier,
            $this->mergeWhen($supplier, fn() => [
                'supplier' => $supplier->name_rz
            ]),
            'idticket' => $this->idticket,
            'idpayment_voucher' => $this->idpayment_voucher,
            $this->mergeWhen($paymentVoucher, fn() => [
                'status_voucher' => $paymentVoucher->status
            ]),
            'type' => $this->type,
            'type_name' => $this->getTypeName($this->type),
            'charge' => $this->charge,
            'payment' => $this->payment,
            'balance'   => $this->balance,
            'idway_to_pay' => $this->idway_to_pay,
            $this->mergeWhen($wayToPay, fn() => [
                'way_to_pay' => $wayToPay->name
            ]),
            'gloss' => $this->gloss,
            'idcompany' => $this->idcompany,
            'iduser' => $this->iduser,
            'status' => $this->status,
            'created_at' => $this->created_at ? Carbon::parse($this->created_at)->format('Y-m-d H:i:s') : '',
            'n_document' => $this->getNDocument($this->type),
            'url_document' => $this->getURLDocument($this->type),
        ];
    }

    private function getTypeName($type)
    {
        $types = [
            '1' => 'Factura emitida',
            '2' => 'Abono/Anticipo recibido',
            '3' => 'Pago de factura',
            '4' => 'Anulación de pago',
            '5' => 'Nota de credito',
            '6' => 'Saldo a favor'
        ];

        return $types[$type] ?? '';
    }

    private function getNDocument($type)
    {
        $n_document     = '';
        $ticket         = $this->relationLoaded('ticket') ? $this->ticket : null;
        $paymentVoucher = $this->relationLoaded('paymentVoucher') ? $this->paymentVoucher : null;

        switch ($type) {
            case '1':
            case '5':
                $n_document = $ticket ? $ticket->folio : '';
                break;

            case '2':
            case '3':
                $n_document = $paymentVoucher ? $paymentVoucher->correlative : '';
                break;

            default:
                break;
        }

        return $n_document;
    }

    private function getURLDocument($type)
    {
        $url            = '';
        $ticket         = $this->relationLoaded('ticket') ? $this->ticket : null;
        $paymentVoucher = $this->relationLoaded('paymentVoucher') ? $this->paymentVoucher : null;

        switch ($type) {
            case '1':
                $url = $ticket ? $ticket->url_invoice : '';
                break;

            case '2':
            case '3':
            case '4':
                $url = $paymentVoucher ? $paymentVoucher->url_document : '';
                break;

            default:
                break;
        }

        return $url;
    }
}
