<?php

namespace App\Http\Resources;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Http\Resources\Json\JsonResource;

class IntakeCartResource extends JsonResource
{
    /**
     * Transform the resource into an array.
     *
     * @return array<string, mixed>
     */
    public function toArray(Request $request): array
    {
        $supplier = $this->relationLoaded('supplier') ? $this->whenLoaded('supplier') : null;
        $details = $this->relationLoaded('details') ? $this->whenLoaded('details') : null;
        $company = $this->relationLoaded('company') ? $this->whenLoaded('company') : null;

        return [
            'id' => $this->id,
            'iduser_created' => $this->iduser_created,
            'idsupplier' => $this->idsupplier,
            $this->mergeWhen($supplier, fn() => [
                'supplier' => $supplier->name_rz
            ]),
            'idcompany' => $this->idcompany,
            $this->mergeWhen($company, fn() => [
                'company'           => $company->business_name,
                'rut_company'       => $company->rut,
                'address_company'   => $company->address,
                'giro_company'      => $company->giro,
                'phone_company'     => $company->phone
            ]),
            'status_entry' => $this->status_entry,
            'status' => $this->status,
            $this->mergeWhen($details, fn() => [
                'details' => IntakeCartDetailResource::collection($details)
            ]),
            'created_at' => $this->created_at ? Carbon::parse($this->created_at)->format('Y-m-d H:i:s') : '',
            'date_format' => Carbon::parse($this->created_at)->isoFormat('D [de] MMMM [de] YYYY'),
        ];
    }
}
