<?php

namespace App\Imports;

use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\ToCollection;
use Maatwebsite\Excel\Concerns\WithHeadingRow;
use App\Models\Supplier;
use App\Models\ClientAddress;
use App\Models\District;

class ClientsDataImport implements ToCollection, WithHeadingRow
{
    protected $idcompany;
    protected $limit;

    public function __construct($params)
    {
        $this->idcompany = $params['idcompany'];
        $this->limit = $params['limit'];
    }

    public function collection(Collection $clients)
    {
        $type = "1";
        $idcompany = $this->idcompany;

         # Limitar la cantidad de filas que se procesan
        $clients = $clients->take($this->limit);

        # Obtener clientes existentes previamente (en memoria)
        $existingClients = Supplier::where('idcompany', $idcompany)->active()->pluck('rut')->toArray();

        # Obtener distritos con su jerarquía (en memoria)
        $districts = District::with(['province.department.country'])
        ->get()
        ->keyBy(fn($item) => $this->normalizeString($item->name));
    

        $dataClients = [];
        $dataAddresses = [];

        foreach ($clients as $client) {
           
            # Validamos que el cliente no exista
            if (in_array($client['rut'], $existingClients)) {
                continue;
            }

            # Adjuntamos data para insertar clientes
            $dataClients[] = [
                'rut'               => $client['rut'],
                'name_rz'           => $client['nombre'],
                'fantasy_name'      => $client['nombre'],
                'giro'              => $client['giro'],
                'phone'             => $client['fono'],
                'type'              => $type,
                'legal_nature'      => "39",
                'idsegment'         => 3,
                'idcompany'         => $idcompany,
                'created_at'        => now(),
                'updated_at'        => now(),
            ];

            # Procesamos direcciones
            $comuna_normalizada = $this->normalizeString($client['comuna']);

            $district = $districts->get($comuna_normalizada);
            $province = $district->province ?? null;
            $department = $province->department ?? null;
            $country = $department->country ?? null;

            $dataAddresses[] = [
                'rut'        => $client['rut'],
                'address'    => $client['direccion'] ?? null,
                'idcountry'  => $country->id ?? null,
                'idprovince' => $province->id ?? null,
                'iddistrict' => $district->id ?? null,
                'idcompany'  => $idcompany,
                'created_at' => now(),
                'updated_at' => now(),
            ];
        }

        # Insertar datos masivos
        if (!empty($dataClients)) {
            Supplier::insert($dataClients);

            # Mapear IDs de clientes insertados
            $suppliers = Supplier::whereIn('rut', array_column($dataClients, 'rut'))
                ->pluck('id', 'rut')
                ->toArray();

            foreach ($dataAddresses as &$address) {
                $address['idclient'] = $suppliers[$address['rut']] ?? null;
                unset($address['rut']); # Eliminamos el campo 'rut' ya que no es necesario para ClientAddress
            }

            ClientAddress::insert($dataAddresses);
        }
    }

    private function normalizeString($string)
    {
        $string = trim(mb_strtolower($string, 'UTF-8')); // Convertir a minúsculas y eliminar espacios
        $string = preg_replace('/\s+/', ' ', $string); // Eliminar espacios extra
        $string = str_replace(
            ['á', 'é', 'í', 'ó', 'ú', 'ñ'], 
            ['a', 'e', 'i', 'o', 'u', 'n'], 
            $string
        ); // Reemplazar tildes y ñ
        return $string;
    }

}
