<?php

namespace App\Imports;

use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\ToCollection;
use Maatwebsite\Excel\Concerns\WithHeadingRow;
use App\Models\StockProduct;
use App\Models\Store;
use App\Models\Product;

class StockProductDataImport implements ToCollection
{
    public function collection(Collection $rows)
    {
        // Remover las primeraa fila (encabezados) si existe
        $rows = $rows->slice(2);

        // Obtener datos únicos de bodegas y productos basados en posiciones
        $storeCodes = $rows->pluck(1)->unique(); // Columna B (CODIGO BODEGA)
        $productCodes = $rows->pluck(3)->unique()->map(function ($code) { // Columna D (CODIGO PRODUCTO)
            return (int) $code; // Convertir a entero
        });

        // Consultar bodegas y productos existentes
        $stores = Store::whereIn('code', $storeCodes)
            ->where('idcompany', request('idcompany'))
            ->get()
            ->keyBy('code');

        $products = Product::whereIn('code_system', $productCodes)
            ->where('idcompany', request('idcompany'))
            ->get()
            ->keyBy('code_system');
        
        $dataInsert = [];
        $newStores = [];

        foreach ($rows as $row) {
            $storeCode = $row[1]; // Columna B
            $storeName = $row[2]; // Columna C
            $productCode = (int) $row[3]; // Columna D
            $stock = $row[4]; // Columna E

            // Crear nueva bodega si no existe
            if (!isset($stores[$storeCode])) {
                $newStores[] = [
                    'code' => $storeCode,
                    'name' => $storeName,
                    'description' => 'Bodega creada automáticamente',
                    'idcompany' => request('idcompany'),
                ];
            }

            // Agregar datos de stock si el producto existe
            if (isset($products[$productCode])) {
                $dataInsert[] = [
                    'stock_erp' => $stock,
                    'stock_system' => $stock,
                    'registration_date' => now(),
                    'last_update' => now(),
                    'idproduct' => $products[$productCode]->id,
                    'idwarehouse' => $stores[$storeCode]->id ?? null,
                    'idcompany' => request('idcompany')
                ];
            }
        }

        // Crear nuevas bodegas en un solo paso
        if (!empty($newStores)) {
            Store::insert($newStores);

            // Consultar nuevamente las bodegas para incluir las recién creadas
            $stores = Store::whereIn('code', $storeCodes)
                ->where('idcompany', request('idcompany'))
                ->get()
                ->keyBy('code');
        }

        foreach ($dataInsert as &$data) {
            if (isset($stores[$data['idwarehouse']])) {
                $data['idwarehouse'] = $stores[$data['idwarehouse']]->id;
            }
        }

        // Eliminar filas inválidas solo si `idwarehouse` sigue siendo nulo
        $dataInsert = array_filter($dataInsert, function ($data) {
            return $data['idwarehouse'] !== null;
        });

        // Insertar datos de stock
        if (!empty($dataInsert)) {
            StockProduct::insert($dataInsert);
        }
    }
}
