<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasOne;

class AccountReceivableDet extends Model
{
    use HasFactory;

    protected $table = "tbl_account_receivable_det";

    protected $fillable = [
        'idaccount_receivable',
        'idpayment_voucher',
        'amount',
        'idcompany',
        'status'
    ];

    protected $hidden = [
        'created_at',
        'updated_at'
    ];

    # Relaciones
    public function accountReceivable(): HasOne
    {
        return $this->hasOne(AccountReceivable::class, 'id', 'idaccount_receivable');
    }

    public function paymentVoucher(): HasOne
    {
        return $this->hasOne(PaymentVoucher::class, 'id', 'idpayment_voucher');
    }

    # Query scopes
    public function scopeActiveForID($query, $id)
    {
        return $query->where('id', $id)->where('status', 1);
    }

    public function scopeActive($query)
    {
        return $query->where('status', 1);
    }

    public function scopeCompany($query)
    {
        return $query->when(
            request('idcompany'),
            fn($q) => $q->where('idcompany', request('idcompany'))
        );
    }

    # Filtros
    public function scopeFilters($query)
    {
        $query->when(
            request('idaccount_receivable'),
            fn($query) => $query->where('idaccount_receivable', request('idaccount_receivable'))
        );

        $query->when(
            request('idpayment_voucher'),
            fn($query) => $query->where('idpayment_voucher', request('idpayment_voucher'))
        );

        $query->company();

        $query->when(
            request('status'),
            fn($query) => $query->where('status', request('status'))
        )->when(
            !request('status'),
            fn($query) => $query->where('status', 1)
        );
    }
}
