<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasOne;

class IntakeCartDetail extends Model
{
    use HasFactory;

    protected $table = "tbl_intake_cart_detail";

    protected $fillable = [
        'idintake_cart',
        'idproduct',
        'quantity',
        'status'
    ];

    protected $hidden = [
        'created_at',
        'updated_at'
    ];

    # Relaciones
    public function intakeCart(): HasOne
    {
        return $this->hasOne(IntakeCart::class, 'id', 'idintake_cart');
    }

    public function product(): HasOne
    {
        return $this->hasOne(Product::class, 'id', 'idproduct');
    }

    # Query scopes
    public function scopeActiveForID($query, $id)
    {
        return $query->where('id', $id)->where('status', 1);
    }

    public function scopeActive($query)
    {
        return $query->where('status', 1);
    }

    public function scopeCompany($query)
    {
        return $query->when(
            request('idcompany'),
            fn($query) => $query->whereHas(
                'intakeCart',
                fn($q) => $q->where('idcompany', request('idcomany'))
            )
        );
    }

    # Filtros
    public function scopeFilters($query)
    {
        $query->when(
            request('idintake_cart'),
            fn($query) => $query->where('idintake_cart', request('idintake_cart'))
        );

        $query->when(
            request('idproduct'),
            fn($query) => $query->where('idproduct', request('idproduct'))
        );

        $query->company();

        $query->when(
            request('status'),
            fn($query) => $query->where('status', request('status'))
        )->when(
            !request('status'),
            fn($query) => $query->where('status', 1)
        );
    }
}
