<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;

class Notifications extends Model
{
    use HasFactory;
    protected $table = "tbl_notifications";
    protected $primaryKey = "id";
    protected $fillable = [
        'idsender',
        'message_title',
        'message_body',
        'data_json',
        'date_sent',
        'idcompany',
        'status'
    ];

    public function sender(): HasOne
    {
        return $this->hasOne(User::class, 'id', 'idsender');
    }

    public function company(): HasOne
    {
        return $this->hasOne(Company::class, 'id', 'idcompany');
    }

    public function userrelation(): HasMany
    {
        return $this->hasMany(DetailUserNotifications::class, 'idnotification', 'id');
    }

     # Query scope
     public function scopeActive($query)
     {
         return $query->where('status', 1);
     }

     public function scopeActiveForID($query, $id)
    {
        return $query->where('id', $id)->where('status', 1);
    }

    public function scopeCompany($query)
    {
        return $query->when(
            request('idcompany'),
            fn ($query) => $query->where('idcompany', request('idcompany'))
        );
    }

    # Filtros
    public function scopeFilters($query)
    {
        #Filtro de idcompany
        $query->company();

        #Filtro de estados
        $query->when(
            request('status'),
            fn ($query) => $query->where('status', request('status'))
        )->when(
            !request('status'),
            fn ($query) => $query->where('status', 1)
        );
    }

}
