<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;

class Route extends Model
{
    use HasFactory;

    protected $table = "tbl_route";

    protected $fillable = [
        'date',
        'n_route',
        'iduser_created',
        'patente',
        'driver',
        'observations',
        'idcompany',
        'status_closure',
        'status'
    ];

    protected $hidden = [
        'created_at',
        'updated_at'
    ];

    #Relaciones
    public function user(): HasOne
    {
        return $this->hasOne(User::class, 'id', 'iduser_created');
    }

    public function company(): HasOne
    {
        return $this->hasOne(Company::class, 'id', 'idcompany');
    }

    public function dispatchGuides(): HasMany
    {
        return $this->hasMany(DispatchGuide::class, 'idroute', 'id');
    }

    # Query scopes
    public function scopeActiveForID($query, $id)
    {
        return $query->where('id', $id)->where('status', 1);
    }

    public function scopeActive($query)
    {
        return $query->where('status', 1);
    }

    public function scopeCompany($query)
    {
        return $query->when(
            request('idcompany'),
            fn($query) => $query->where('idcompany', request('idcompany'))
        );
    }

    # Filtros
    public function scopeRouteFilters($query)
    {
        # Filtro de fecha exacta
        $query->when(
            request('date'),
            fn($query) => $query->whereRaw('DATE(date) = ?', [request('date')])
        );

        #Filtro de id
        $query->when(
            request('id'),
            fn($query) => $query->where('id', request('id'))
        );

    }

}
