<?php

namespace App\Services;

use App\Exports\CartolaExport;
use App\Http\Resources\CartolaResource;
use App\Models\Cartola;
use App\Models\Supplier;
use App\Traits\HasResponse;
use Maatwebsite\Excel\Facades\Excel;
use Barryvdh\DomPDF\Facade\Pdf;

/**
 * Class CartolaService
 * @package App\Services
 */
class CartolaService
{
    use HasResponse;

    public function list($params, $withPagination)
    {
        try {
            $data = Cartola::filters();

            $data = !empty($withPagination)
                ? $data->paginate($withPagination['perPage'], page: $withPagination['page'])
                : $data->get();

            $paginationTotal = null;
            if (!empty($withPagination)) $paginationTotal = $data->total();

            $data = CartolaResource::collection($data->load(
                'supplier',
                'ticket',
                'wayToPay',
                'company',
                'user',
                'paymentVoucher'
            ));

            return $this->successPaginationResponse('Lectura exitosa.', $paginationTotal, $data);
        } catch (\Throwable $th) {
            throw $th;
        }
    }

    public function downloadExcel($params)
    {
        try {
            # Validar existencia del cliente
            $supplier = Supplier::activeForID($params['idsupplier'])->first();
            if (!$supplier) return $this->errorResponse('Cliente no válido.', 400);

            $details = Cartola::filters()
                ->get()
                ->load(
                    'ticket',
                    'wayToPay',
                    'paymentVoucher'
                );

            $data = [
                'start_date' => $params['startDate'] ?? '',
                'end_date' => $params['endDate'] ?? '',
                'name_rz'  => $supplier->name_rz,
                'rut'      => $supplier->rut,
                'details'  => $details
            ];

            $name = str_replace(' ', '_', $supplier->name_rz);
            $fileName = $name . "_cartola.xlsx";
            $baseUrl = url('/');

            Excel::store(new CartolaExport($data), $fileName, 'public');

            return $this->successResponse('OK', $baseUrl . '/storage/' . $fileName);
        } catch (\Throwable $th) {
            return $this->externalError('durante la descarga.', $th->getMessage());
        }
    }

    public function downloadPDF($params)
    {
        try {
            # Validar existencia del cliente
            $supplier = Supplier::activeForID($params['idsupplier'])->first();
            if (!$supplier) return $this->errorResponse('Cliente no válido.', 400);

            $details = Cartola::filters()
                ->get()
                ->load(
                    'ticket',
                    'wayToPay',
                    'paymentVoucher'
                );

            $data = [
                'start_date' => $params['startDate'] ?? '',
                'end_date' => $params['endDate'] ?? '',
                'name_rz'  => $supplier->name_rz,
                'rut'      => $supplier->rut,
                'details'  => $details,
                'types'    =>  [
                    '1' => 'Factura emitida',
                    '2' => 'Abono/Anticipo recibido',
                    '3' => 'Pago de factura',
                    '4' => 'Anulación de pago',
                ]
            ];

            $pdf = Pdf::loadView('pdfs.cartolaPDF', $data)
                ->setPaper('a4', 'landscape');

            $fileName = str_replace(' ', '_', $supplier->name_rz) . '.pdf';
            $filePath = storage_path('app/public/');
            if (!file_exists($filePath)) {
                mkdir($filePath, 0777, true);
            }

            $path = $filePath . $fileName;
            $pdf->save($path);

            $url = asset('storage/' . $fileName);

            return $this->successResponse('PDF generado con éxito.', $url);
        } catch (\Throwable $th) {
            return $this->externalError('durante la descarga del PDF.', $th->getMessage());
        }
    }
}
