<!DOCTYPE html>
<html lang="es">

<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Comprobante de Pago</title>
    <style>
        body {
            font-family: 'Arial', sans-serif;
            margin: 0;
            padding: 0;
            font-size: 12px;
        }

        .content {
            padding: 20px;
        }

        .container {
            width: 100%;
            padding: 20px;
            max-width: 800px;
            margin: 0 auto;
            background: white;
            border-radius: 8px;
            box-shadow: 0 2px 10px rgba(0, 0, 0, 0.1);
        }

        h1,
        h2,
        h3 {
            color: #516E80;
            text-align: center;
            margin-bottom: 10px;
        }

        .section {
            margin-bottom: 20px;
        }

        .section h2 {
            text-align: left;
            font-size: 18px;
            margin-bottom: 10px;
            color: #516E80;
        }

        .section p {
            font-size: 14px;
            margin: 5px 0;
        }

        .table {
            width: 100%;
            border-collapse: collapse;
            margin-top: 10px;
        }

        .table th,
        .table td {
            border: 1px solid #ddd;
            padding: 8px;
            text-align: left;
        }

        .table th {
            background-color: #516E80;
            color: white;
        }

        .table td {
            background-color: #f9f9f9;
        }

        .header,
        .footer {
            text-align: center;
            margin-bottom: 30px;
        }

        .footer {
            position: absolute;
            bottom: 20px;
            width: 100%;
        }

        .signature {
            margin-top: 40px;
            padding-top: 10px;
            text-align: center;
        }

        .signature h2 {
            color: #516E80;
            font-size: 16px;
            margin-bottom: 15px;
        }

        .signature .table {
            width: 60%;
            margin: 0 auto;
            border: none;
        }

        .signature .table td {
            border: none;
            padding: 10px;
        }


        .styled-table td:first-child {
            background-color: #516E80;
            color: white;
            font-weight: bold;
            text-align: left;
            padding: 8px;
            width: 30%;
        }

        .styled-table td:nth-child(2) {
            width: 70%;
        }
    </style>
</head>

<body>
    @php
        if (!function_exists('numberToWords')) {
            function numberToWords($number)
            {
                $formatter = new NumberFormatter('es', NumberFormatter::SPELLOUT);
                return ucfirst($formatter->format($number));
            }
        }
    @endphp

    <div class="container">

        <h1>
            COMPROBANTE DE PAGO
        </h1>

        <div class="section">
            <table class="table styled-table">
                <tr>
                    <td colspan="2" style="text-align: center">
                        Información general
                    </td>
                </tr>
                <tr>
                    <td>Correlativo:</td>
                    <td>{{ $data->correlative ?? 'N/A' }}</td>
                </tr>
                <tr>
                    <td>RUT:</td>
                    <td>{{ $data->rut }}</td>
                </tr>
                <tr>
                    <td>Razón Social:</td>
                    <td>{{ $data->supplier }}</td>
                </tr>
                <tr>
                    <td>Método de pago:</td>
                    <td>{{ $data->way_to_pay ?? 'N/A' }}</td>
                </tr>
                @if ($data->idway_to_pay == 18)
                    <tr>
                        <td>Serie:</td>
                        <td>{{ $data->series ?? 'N/A' }}</td>
                    </tr>
                    <tr>
                        <td>Fecha límite de cobro:</td>
                        <td>{{ $data->document_date ?? 'N/A' }}</td>
                    </tr>
                @endif;
                <tr>
                    <td>Monto:</td>
                    <td>$ {{ number_format($data->amount, 0, ',', '.') ?? 'N/A' }}</td>
                </tr>
                <tr>
                    <td>Fecha de pago</td>
                    <td>{{ \Carbon\Carbon::parse($data->created_at)->format('d/m/Y H:i') }}</td>
                </tr>
            </table>
        </div>

        <div class="section">
            <table class="table">
                <thead>
                    <tr>
                        <th>N° Folio</th>
                        <th>Emisión</th>
                        <th>Vencimiento</th>
                        <th>Total facturado</th>
                        <th>Monto pagado</th>
                    </tr>
                </thead>
                <tbody>
                    @foreach ($data->accountReceivableDets as $detail)
                        <tr>
                            <td>{{ $detail->account_receivable->ticket->folio ?? 'N/A' }}</td>
                            <td>{{ \Carbon\Carbon::parse($detail->account_receivable->ticket->created_at)->format('d/m/Y H:i') }}
                            </td>
                            <td>{{ \Carbon\Carbon::parse($detail->account_receivable->deadline)->format('d/m/Y') }}
                            </td>
                            <td>$ {{ number_format($detail->account_receivable->total, 0, ',', '.') }}</td>
                            <td>$ {{ number_format($detail->amount, 0, ',', '.') }}</td>
                        </tr>
                    @endforeach
                </tbody>
                <tr>
                    <td colspan="4"
                        style="background-color: #516E80; text-align: center; color: #fff; font-weight: bolder">
                        TOTAL
                    </td>
                    <td style="font-weight: bolder">$ {{ number_format($data->amount, 0, ',', '.') }}
                    </td>
                </tr>
            </table>
        </div>

        <div class="section">
            <table class="table styled-table">
                <tr>
                    <td>Son:</td>
                    <td>{{ numberToWords($data->amount) }} pesos</td>
                </tr>

            </table>
        </div>

        <div class="section">
            <table class="table styled-table">
                <tr>
                    <td>RUT:</td>
                    <td></td>
                </tr>
                <tr>
                    <td>Nombre:</td>
                    <td></td>
                </tr>
                <tr>
                    <td>Firma:</td>
                    <td style="height: 50px"></td>
                </tr>
                <tr>
                    <td colspan="2" style="text-align: center">Recibí
                        conforme
                    </td>
                </tr>
            </table>
        </div>

        <div class="footer">
            Generado por el sistema de Facturación | <a href="mailto:soporte@sistema.com">soporte@sistema.com</a>
        </div>
    </div>
</body>

</html>
